@@ -19,6 +19,16 @@ import gzip
 
 warnings.simplefilter(action="ignore", category=FutureWarning)
 
+taxo_ranks = [
+    'strain',
+    'species',
+    'genus',
+    'family',
+    'order',
+    'class',
+    'phylum',
+    'superkingdom'
+]
 
 def count_reads_taxid(read_taxid_dict):
     """Returns number of reads matching TAXID
@@ -103,10 +113,12 @@ def taxid_to_lineage_single(taxid_items, taxid_info_dict, taxo_db):
         sciname = taxon.name
         rank = taxon.rank
         lineage = taxon.rank_name_dictionary
+        taxid_lineage = taxon.taxid_lineage
     except Exception:
         sciname = "unclassified sequences"
         rank = "NA"
         lineage = "NA"
+        taxid_lineage = "NA"
 
     taxid_info_dict[taxid] = {
         "name": sciname,
@@ -114,6 +126,7 @@ def taxid_to_lineage_single(taxid_items, taxid_info_dict, taxo_db):
         "count_taxon": read_count_taxon,
         "count_descendant": read_count_descendant,
         "lineage": lineage,
+        "lineage_taxid": taxid_lineage,
     }
 
 
@@ -146,25 +159,38 @@ def taxid_to_lineage(taxid_count_dict, output, process, nb_steps, taxo_db):
     )
 
     df = pd.DataFrame(taxid_info_dict).transpose()
-    df["lineage"] = (
-        df["lineage"]
-        .astype(str)
-        .str.replace("[\[\]\{\}]", "")
-        .str.replace(", ", " || ")
-        .str.replace("'", "")
-    )
-    df.sort_values("count_descendant", inplace=True, ascending=False)
-    df.to_csv(f"{output}.csv", index_label="TAXID")
+    if df.shape[0] > 0:
+        df["lineage"] = (
+            df["lineage"]
+            .astype(str)
+            .str.replace("[\[\]\{\}]", "")
+            .str.replace(", ", " || ")
+            .str.replace("'", "")
+        )
+        df.sort_values("count_descendant", inplace=True, ascending=False)
+        df.to_csv(f"{output}.csv", index_label="TAXID")
+    else:
+        logging.info("WARNING: no reads fulfilled the selected identity "
+                     f"threshold. Skip writing results to {output}.csv")
 
     with open(f"{output}.json", "w") as write_file:
         json.dump(taxid_info_dict, write_file)
     logging.info(
-        f"Step {7 if nb_steps == 7 else 8 }/{nb_steps}: writing sam2lca results:\n* JSON to {output}.json\n* CSV to {output}.csv"
+        f"Step {7 if nb_steps == 7 else 8 }/{nb_steps}: writing sam2lca results:\n* JSON to {output}.json"
     )
+    if df.shape[0] > 0:
+        logging.info(f"* CSV to {output}.csv")
 
     return taxid_info_dict
 
 
+def filter_taxid_info_dict(taxid_info_dict, rank, minread):
+    taxid_info_dict_filtered = dict()
+    for entry in taxid_info_dict:
+        if taxid_info_dict[entry]['rank'] == rank and taxid_info_dict[entry]['count_descendant'] >= minread:
+            taxid_info_dict_filtered[entry] = taxid_info_dict[entry]
+    return taxid_info_dict_filtered
+
 def get_db_connection(db_path):
     return rocksdb.DB(db_path, opts=rocksdb.Options(), read_only=True)
 
